/* Teamraum Styles for Seantis Dir Events */

/* directory search header */
.portaltype-seantis-dir-events-directory #directorySearch {
    background-color: #efefef;
    border-color: #ddd;border-width: 0 1px 1px 1px;
    padding: 1em 1.5em;
}

/* header description */
#landingPageIntro {
    margin-bottom: 25px;
}

/* event directory columns */
.event-navigation, #directoryResultList, .event-sidebar, .event-view {
    float: left;
    -webkit-box-sizing: border-box;
       -moz-box-sizing: border-box;
         -o-box-sizing: border-box;
            box-sizing: border-box;
}

.event-navigation, #directoryResultList {
    padding-right: 25px;
}

.event-navigation {
    width: 20%;
}

#directoryResultList {
    width: 45%;
}

.event-sidebar {
    width: 35%;   
}
.event-view {
    width: 65%;   
}

@media (max-width: 900px) {
    .event-navigation, #directoryResultList, .event-sidebar, .event-view {
         float: none;
         width: 100%;
         padding: 0;  
    }
}

/* event highlighting */
#directoryResultList .groupSelection {
    -webkit-box-shadow: none !important;
    -moz-box-shadow: none !important;
    box-shadow: none !important;
}
#directoryResultList .groupSelection h3 {
    background-color: #fff7b5;
    border-radius: 1px;
}
/* event navigation */

.event-navigation ul {
    margin: 0 !important;
    margin-bottom: 5px !important;
}

.event-navigation ul li {
    margin: 0 !important;
    list-style: none;
}

/* event navigation links */
.event-navigation ul a {
    display: block;
    background-color: #f8f8f8;
    padding: 2px 4px;
    border-radius: 3px;
}

#content .event-navigation ul a:hover,
.event-navigation ul a.selected {
    text-decoration: none !important;
    color: white;
    background-color: #004373;
    background-image: -moz-linear-gradient(top, #005fa2, #004373);
    background-image: -ms-linear-gradient(top, #005fa2, #004373);
    background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#005fa2), to(#004373));
    background-image: -webkit-linear-gradient(top, #005fa2, #004373);
    background-image: -o-linear-gradient(top, #005fa2, #004373);
    background-image: linear-gradient(top, #005fa2, #004373);
    filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#005fa2', endColorstr='#004373', GradientType=0);
}

/* event results */
.directoryResult {
    border: none !important;
    border-bottom: 1px solid #ddd !important;
    margin-left: 23px;
    position: relative;
}

.directoryResult:last-child {
    border-bottom: none !important;   
}

.directoryMarker {
    position: absolute;
    left: -26px !important;
    top: -2px;   
}

/* admin controls */
.event-workflow {
    float: left;
    margin-top: 5px;
    font-weight: bold;
}

.event-state {
    color: #555;   
}

.event-actions {
    font-weight: bold;   
}

.event-actions a {
    display: block;   
}

/* single result */
.eventgroup-title {
    margin: 6px auto 2px auto;
    font-size: 1.4em;
    color: #205c90;
    font-weight: bold;
    text-align: right;
    border: none !important;
    border-bottom: #205c90 solid 1px !important;
    padding-bottom: 2px;
}

.event-text h3 {
    font-size: 1.4em !important;   
}

.event-text .locality {
    font-weight: bold;
}

.event-text {
    color: #333;
    font-size: 12px;
    margin-bottom: 5px;
}

.event-image {
    float: right;
    background-color: #efefef;
    padding: 15px;
    margin: 0 0 5px 5px;
}

.directoryInfoText {
    padding-left: 0 !important;   
}

/* directory map */
#directoryResultMap {
    margin-top: 26px;
    margin-bottom: 26px;
}

#default-cgmap {
    height: 300px;      
}

/* tags */
#content .eventtags a,
#content .eventtags a:hover {
    border: none;
}

.eventtag {
    background-color: #efefef;
    padding: 2px 4px;
    border-radius: 3px;   
}

/* event time */
.timerange, .event-date {
    background-color: #54a954;
    padding: 2px 4px;
    border-radius: 3px;
    float: right;
    text-align: center;
    display: inline-block;
    clear: both;
    min-width: 88px;
    color: white;
}

/* submit event */
.your-event {
    font-size: 1.2em;
    margin-top: 20px;
    margin-bottom: 20px;
    padding: 30px 25px;
    text-align: right;
    border-width: 0;
    color: #444444;
    border-color: #efeda4;
}

#content .your-event a,
#content .your-event a:hover {
    color: #448944;
    text-decoration: underline;
    border-color: white;
    font-weight: bold;
    border-width: 0 0 4px 0;
}

/* event detail head */
.event-date {
    font-size: 1.2em;
    font-weight: bold;
    float: none;
    margin-top: 5px;
    margin-bottom: 5px;
}

.descriptions {
    max-width: 700px;
    margin-right: 50px;
}

.descriptions .short {
    font-weight: bold;
    margin: 5px 0 10px 0;   
}

.descriptions .long {
    color: #333;
    margin: 0 0 25px 0;   
}

/* adjust map on event item view */
#directoryDetails .event-sidebar {
    margin-top: 14px;   
}

/* same with tags */
#directoryDetails .eventtags {
    margin-bottom: 25px;    
}

/* event detail blocks */
.event-details > div {
    margin-bottom: 25px;
    margin-right: 50px;      
}

.event-details h3 {
    font-weight: normal;
    font-size: 1.2em;
    border-bottom: 1px solid #eee;
    margin-bottom: 10px;
    color: black;
}

.event-details ul {
    margin-left: 0 !important;
    list-style: none !important;
}

@media (max-width: 900px) {
    .event-details > div, .descriptions {
        margin-right: 0 !important;
    }
}

/* submit form */
#formfield-form-widgets-cat1,
#formfield-form-widgets-cat2 {
    width: 50%;
    float: left;
    clear: none;  
}
#formfield-form-widgets-cat1 > span,
#formfield-form-widgets-cat2 > span {
    display: block;   
}

#formfield-form-widgets-long_description textarea {
    height: 200px;
}

input.required[type="radio"] {
    visibility: visible !important;
    position: relative;
    top: -2px;
    margin-right: 8px; 
}

input.required[type="radio"]:after {
    display: none !important;   
}

#formfield-form-widgets-submission_start_time input,
#formfield-form-widgets-submission_end_time input,
#formfield-form-widgets-submission_range_start_time input,
#formfield-form-widgets-submission_range_end_time input {
    width: 200px;   
}

#formfield-form-widgets-submission_days .option {
    float: left;
    margin-right: 0.75em;
}

#formfield-form-widgets-submission_days .option input {
    margin-right: 0;   
}

/* event preview form */
.event-preview-form {
    margin-top: 10px;
}

/* 

    the recurrence widget looks awful by default
    
*/

/* the headers are too big and prominent */
.riform h1, .riform h2 {
    color: #333 !important;
    font-size: 14px !important;
}

/* the lower headers have different distances to their borders */
.rioccurancesheader {
    height: 23px;
}

/* the font is too small */
.riform {
    font-size: 13px !important;   
}

/* the date field in the recurrence section is badly aligned */
#riformfields .date {
    width: 85px !important;
    position: relative;
    left: 5px;
    top: -1px;
}

/* the labels are too small (text is wrapped) and they are too heavy */
.riform .label {
    width: 150px !important;
    font-weight: normal !important;
}

/* the select field looks off */
div.riform select {
    position: relative;
    top: -2px;   
}

/* the fields are smashed too close together */
#riformfields .label, #riformfields .field {
    margin-top: 8px;
}

#rirangeoptions div {
    margin-bottom: 10px;   
}

/* the trash symbols are scattered all over the place */
.rioccurrences .occurrence .action {
    position: absolute;
    right: 6px;
}

.rioccurrences .occurrence .action a {
    float: none;
    display: inline-block;
    position: relative;
    top: -3px;
}

/* the refresh button is badly aligned */
.rirefreshbutton {
    position: relative;
    left: 5px;
    top: -8px;   
}

/* the add date button is weirdly shown under the date */
.riform #adddate {
    margin-top: 10px;
    margin-bottom: 10px;
    width: 290px !important;
}

/* the small 'beginning of recurrence'-label is unreadable and positioned awfully*/
.riform .start .rlabel {
    color: #37911b !important;   
    font-weight: normal !important;
    font-size: 13px !important;
    float: right;
    position: relative;
    left: 4px;
}

/* the first occurrence's top border clashes with the header */
.rioccurrences .occurrence:hover {
    border-color: white !important;
    background-color: #efefef;
}

/* the batching is, you guessed it, way too small to be readable */
.riform .batching {
    font-size: 13px !important;
}

/* the cancel button animates away on hover (I'm not making this up) */
input.ricancelbutton {
    background-position: 0 !important
}

/*

    end of recurrence widget fixes

*/


/* wizard */
#content form.seantis-form-navigation ul.wizardSteps {
    padding-bottom: 1em;
}
#content form.seantis-form-navigation ul.wizardSteps li:before {
    content : '';
}
#content form.seantis-form-navigation ul.wizardSteps li a:before {
    content : '〉';
}
#content form.seantis-form-navigation ul.wizardSteps li:first-child a:before {
    content : '';
}
